#include <18F2550.h>
#fuses HSPLL,NOWDT,MCLR,NOPROTECT,NOLVP,NODEBUG,USBDIV,PLL5,CPUDIV1,VREGEN
#use delay(clock=48000000)

#include <stdlib.h>
#include <usb_cdc.h>
#include <usb_bootloader.h>
#include <internal_eeprom.c>
#include "Flex_LCD420.c"
#include "ADE7753.h"

static int1 Update, Display, Hold;

#int_TIMER1 
void TIMER1_isr(void) //10mS
{
   static int8 Count;
   int1 key1, key2;
   static int1 old1, old2;
   
   set_timer1(50536);
   key1 = input(PIN_A1);
   key2 = input(PIN_A2);
   if (old1 & !key1) Display = Display^1;
   if (old2 & !key2) Hold = Hold^1;
   old1 = key1;
   old2 = key2;
         
   if (Count++==60)
   {
      Count = 0;
      Update = 1;
   }
}

void Calibration()
{
   float Recv;
   char c;
   int32 Sq;
   
   printf(usb_cdc_putc, "Calibration Mode...\r\nPress 1 to Remove Current Offset\r\nPress 2 to Enter new Voltage\r\nPress 3 to Enter new Current\r\n");
   printf(usb_cdc_putc, "Press 4 to Enter new Active Power\r\nPress 5 to Enter new Reactive Power\r\nPress 6 to Enter new Apparent Power\r\n");  
   printf(usb_cdc_putc, "Any other key cancels Calibration Mode !\r\n");
   c=usb_cdc_getc();
   
   if (c=='1')
   {
       Read16(0x0C);
       //attente d'un passage  0 pour effectuer la mesure (ou timeout)
       while(!(Read16(0x0B)&0x1010));  
       
       Ioff = Read24(0x16);
       write_int32_eeprom(20,Ioff);
       printf(usb_cdc_putc, "\r\nOffset Removed !!!\r\n"); 
   }
   
   if (c=='2')
   {
      printf(usb_cdc_putc, "Actual Gain : %1.6f\r\nActual Voltage : %f\r\n",Vgain,Vrms);
      printf(usb_cdc_putc, "Enter Now Actual Real Voltage > ");
      Recv = get_float_usb();
      if (Recv!=0.0 && Vraw!=0) 
      {
          printf(usb_cdc_putc, "\r\nCalibration Complete\r\n");
          Vgain = 1000.0*Recv/(float)(Vraw);
          write_float_eeprom(0,Vgain);
      }
      else printf(usb_cdc_putc, "\r\nCalibration Canceled\r\n");
   }
      
   if (c=='3')
   {
      printf(usb_cdc_putc, "Actual Gain : %1.6f\r\nActual Current : %f\r\n",Igain,Irms);
      printf(usb_cdc_putc, "Enter Now Actual Real Current > ");
      Recv = get_float_usb();
      if (Recv!=0.0 && Iraw!=0) 
      {
          printf(usb_cdc_putc, "\r\nCalibration Complete\r\n");
          Igain = 1000.0*Recv/(float)(Iraw);
          write_float_eeprom(4,Igain);
      }
      else printf(usb_cdc_putc, "\r\nCalibration Canceled\r\n");
   }
   
   if (c=='4')
   {
      printf(usb_cdc_putc, "Actual Gain : %1.6f\r\nActual Active power : %f\r\n",Pgain,P);
      printf(usb_cdc_putc, "Enter Now Actual Real Active Power > ");
      Recv = get_float_usb();
      if (Recv!=0.0 && Praw!=0) 
      {
          printf(usb_cdc_putc, "\r\nCalibration Complete\r\n");
          Pgain = 10000*Recv/(float)(Praw)*Period;
          write_float_eeprom(8,Pgain);
      }
      else printf(usb_cdc_putc, "\r\nCalibration Canceled\r\n");
   }
   
   if (c=='5')
   {
      printf(usb_cdc_putc, "Actual Gain : %1.6f\r\nActual Reactive power : %f\r\n",Qgain,Q);
      printf(usb_cdc_putc, "Enter Now Actual Real Reactive Power > ");
      Recv = get_float_usb();
      if (Recv!=0.0 && Qraw!=0) 
      {
          printf(usb_cdc_putc, "\r\nCalibration Complete\r\n");
          Qgain = 10000*Recv/(float)(Qraw)*Period;
          write_float_eeprom(12,Qgain);
      }
      else printf(usb_cdc_putc, "\r\nCalibration Canceled\r\n");
   }
   
   if (c=='6')
   {
      printf(usb_cdc_putc, "Actual Gain : %1.6f\r\nActual Apparent power : %f\r\n",Sgain,S);
      printf(usb_cdc_putc, "Enter Now Actual Real Apparent Power > ");
      Recv = get_float_usb();
      if (Recv!=0.0 && Sraw!=0) 
      {
          printf(usb_cdc_putc, "\r\nCalibration Complete\r\n");
          Sgain = 10000*Recv/(float)(Sraw)*Period;
          write_float_eeprom(16,Sgain);
      }
      else printf(usb_cdc_putc, "\r\nCalibration Canceled\r\n");
   }
}

void main()
{
   char c;
   
   setup_timer_1(T1_INTERNAL|T1_DIV_BY_8);
   setup_spi(SPI_MASTER | SPI_CLK_DIV_64 | SPI_L_TO_H ); //Mode 1 Clk/64
      
   Vgain = read_float_eeprom(0);
   Igain = read_float_eeprom(4);
   Pgain = read_float_eeprom(8);
   Qgain = read_float_eeprom(12);
   Sgain = read_float_eeprom(16);
   Ioff  = read_int32_eeprom(20);
   
   usb_cdc_init();
   usb_init();
   lcd_init();
   ade_setPGA(1, 1);
   ade_init();
   
   enable_interrupts(INT_TIMER1);
   enable_interrupts(GLOBAL);
   
   printf (lcd_putc,"Waiting ...");
   delay_ms(2500);
   
   while(TRUE)
   {
      //Gestion USB
      usb_task();
      enable_interrupts(GLOBAL);
      if (usb_cdc_connected())
      {
         if (usb_cdc_kbhit()) 
         {
            c=usb_cdc_getc();  
            if (c=='C') Calibration();
            if (c=='V') printf(usb_cdc_putc, "%1.1f\r\n",Vrms);
            if (c=='I') printf(usb_cdc_putc, "%1.3f\r\n",Irms);
            if (c=='F') printf(usb_cdc_putc, "%1.2f\r\n",Freq);
            if (c=='P') printf(usb_cdc_putc, "%1.1f\r\n",P);
            if (c=='Q') printf(usb_cdc_putc, "%1.1f\r\n",Q);
            if (c=='S') printf(usb_cdc_putc, "%1.1f\r\n",S);
            if (c=='o') printf(usb_cdc_putc, "%1.3f\r\n",PF);
         } 
        
      }
      
      //Mesures Puissances
      GetPowerLoop();
      
      //Mesures U/I/F et affichage
      if (Update) //Called Every 600mS
      {
         GetADCValues();
         if (!Hold)
         {
            if (!Display)
            {
               lcd_blank(1);
               printf (lcd_putc,"Urms = %1.1fV",Vrms);
               lcd_blank(2);
               printf (lcd_putc,"Irms = %1.3fA",Irms); 
               lcd_blank(3);
               printf (lcd_putc,"P = %1.1fW",P);
               lcd_blank(4);
               printf (lcd_putc,"Freq = %1.2fHz",Freq);
            }
            else
            {
               lcd_blank(1);
               printf (lcd_putc,"P = %1.1fW",P);
               lcd_blank(2);
               printf (lcd_putc,"S = %1.1fVA",S);
               lcd_blank(3);
               printf (lcd_putc,"PF = %1.3f",PF);
               lcd_blank(4);
               printf (lcd_putc,"Irms = %1.3fA",Irms); 
            }
         }
         lcd_gotoxy(17,4);
         if (Hold)  printf (lcd_putc,"HOLD");
         else       printf (lcd_putc," RUN");  
         
         Update = 0;
      }
   }

}
